#############
# $Id: BamClusterCounts.pm 14686 2011-11-28 14:12:54Z mg8 $
# Created By: gq1
# Last Maintained By: $Author: mg8 $
# Created On: 2011-11-25
# Last Changed On: $Date: 2011-11-28 14:12:54 +0000 (Mon, 28 Nov 2011) $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-pipeline/tags/release-39.0/lib/npg_pipeline/archive/file/BamClusterCounts.pm $

package npg_pipeline::archive::file::BamClusterCounts;

use Moose;
use Carp;
use English qw{-no_match_vars};
use File::Spec;
use Readonly; Readonly::Scalar our $VERSION => do { my ($r) = q$LastChangedRevision: 14686 $ =~ /(\d+)/mxs; $r; };
use XML::LibXML;
use List::MoreUtils qw(uniq);
use npg_qc::autoqc::qc_store;

extends qw{npg_pipeline::archive::file::generation};

Readonly::Scalar our $CLUSTER_COUNTS_SCRIPT => q{npg_pipeline_check_bam_file_cluster_count};

=head1 NAME

npg_pipeline::archive::file::BamClusterCounts

=head1 VERSION

$LastChangedRevision: 14686 $

=head1 SYNOPSIS


  my @job_ids;
  eval {
    my $oClusterCounts = npg_pipeline::archive::file::BamClusterCounts->new({
      run_folder          => $run_folder,
      timestamp           => q{20090709-123456},
      internal_script_bin => q{/software/solexa/bin},
      id_run              => 1234,
      verbose              => 1, # use if you want logging of the commands sent to LSF
    });

    my $arg_refs = {
      required_job_completion  => q{-w'done(123) && done(321)'},
    }

    @job_ids = $oClusterCounts->launch($arg_refs);
  } or do {
    # your error handling here
  };

  my $oClusterCounts = npg_pipeline::archive::file::ClusterCounts->new({
    run_folder => $run_folder,
    verbose    => 1, # use if you want logging of the commands sent to LSF
  });
  $oClusterCounts->run_cluster_count_check();

=head1 DESCRIPTION

This module is responsible for launching a job, and then processing that job, which will check the cluster
counts are what they are expected to be in bam files comparing with summary xml file

=head1 SUBROUTINES/METHODS

=head2 launch

Method launches lsf job commands and returns an array of job ids retrieved

  my @JobIds = $oClusterCounts->launch({
    required_job_completion  => q{-w'done(123) && done(321)'},
  });

=cut

sub launch {
  my ( $self, $arg_refs ) = @_;

  my @job_ids;

  my @positions = $self->positions( $arg_refs );

  if ( ! scalar @positions ) {
    $self->log( q{no positions found, not submitting any jobs} );
    return @job_ids;
  }

  $arg_refs->{array_string} = $self->create_array_string( @positions );

  push @job_ids, $self->submit_bsub_command( $self->_generate_bsub_command( $arg_refs ) );

  return @job_ids;
}

=head2 run_cluster_count_check

method which goes checking the cluster counts

=cut

sub run_cluster_count_check {
  my ( $self ) = @_;

  $self->_bustard_pf_cluster_count();
  $self->_bustard_raw_cluster_count();

  my $check_failed = 0;

  eval {
    $self->_check_bam_cluster_count();
    1;
  } or do {
    $self->log( $EVAL_ERROR );
    $check_failed++;
  };

  if($check_failed){
     croak 'At least one of the cluster count checks failed';
  }

  return 1;
}

has q{position} => (
  isa => q{Int},
  is  => q{ro},
);

#############
# private methods

has q{_bustard_pf_cluster_count} => (
  isa => q{Int},
  is  => q{ro},
  lazy_build => 1,
  writer => q{_set_bustard_pf_cluster_count},
);

sub _build__bustard_pf_cluster_count {
  my ( $self ) = @_;
  return $self->_populate_cluster_counts( q{pf} );
}


has q{_bustard_raw_cluster_count} => (
  isa => q{Int},
  is  => q{ro},
  lazy_build => 1,
  writer => q{_set_bustard_raw_cluster_count},
);

sub _build__bustard_raw_cluster_count {
  my ( $self ) = @_;
  return $self->_populate_cluster_counts( q{raw} );
}

sub _populate_cluster_counts {
  my ( $self, $type ) = @_;

  my $parser = XML::LibXML->new();

  my $xml = $parser->parse_file( $self->bam_basecall_path() . q{/BustardSummary.xml} );

  my %h;

  my $return;

  foreach my $lx ( $xml->getElementsByTagName( q(TileResultsByLane) )->[0]->getElementsByTagName( q(Lane) ) ) {
    my $l = $lx->getElementsByTagName( q(laneNumber) )->[0]->textContent();

    if ( $l != $self->position() ) {
      next;
    }

    foreach my $rx ( $lx->getElementsByTagName( q(Read) ) ) {
      my $r = $rx->getElementsByTagName( q(readNumber) )->[0]->textContent();
      foreach my $tx ( $rx->getElementsByTagName( q(Tile) ) ) {
        foreach my $count ( qw(clusterCountRaw clusterCountPF) ) {
          $h{$l}{$count}{$r} += $tx->getElementsByTagName($count)->[0]->textContent();
        }
      }
    }
  }

  foreach my $l ( sort {$a<=>$b} keys %h ) {

    my @ccpf = uniq values %{ $h{$l}{clusterCountPF} };
    croak q(inconsistent pf counts between reads) if ( @ccpf > 1 );
    $self->_set_bustard_pf_cluster_count( $ccpf[0] );
    if ( $type eq q{pf} ) {
      $return = $ccpf[0];
    }

    my @ccr = uniq values %{ $h{$l}{clusterCountRaw} };
    croak q(inconsistent raw counts between reads) if ( @ccr > 1 );
    $self->_set_bustard_raw_cluster_count( $ccr[0] );
    if ( $type eq q{raw} ) {
      $return = $ccr[0];
    }
  }

  if ( !defined $return ) {
    croak q{Unable to determine a raw and/or pf cluster count};
  }

  return $return;

}

# generates the bsub command
sub _generate_bsub_command {
  my ($self, $arg_refs) = @_;

  my $array_string = $arg_refs->{array_string};
  my $required_job_completion = $arg_refs->{required_job_completion} || q{};
  my $timestamp = $self->timestamp();
  my $solexa_bin = $self->internal_script_bin();
  my $id_run = $self->id_run();

  my $job_name = $CLUSTER_COUNTS_SCRIPT . q{_} . $id_run . q{_} . $timestamp;

  my $archive_out = $self->archive_path() . q{/log};
  my $out_subscript = q{.%I.%J.out};
  my $outfile = File::Spec->catfile($archive_out, $job_name . $out_subscript);

  $job_name = q{'} . $job_name . $array_string . q{'};

  my $job_sub = q{bsub -q } . $self->lsf_queue() . qq{ $required_job_completion -J $job_name -o $outfile '};
  $job_sub .= File::Spec->catfile($solexa_bin, $CLUSTER_COUNTS_SCRIPT);

  $job_sub .= q{ --id_run=} . $id_run;
  $job_sub .= q{ --position=}  . $self->lsb_jobindex();
  $job_sub .= q{ --archive_path=} . $self->archive_path();
  $job_sub .= q{ --bam_basecall_path=} . $self->bam_basecall_path();

  if ( $self->verbose() ) {
    $job_sub .= q{ --verbose};
  }

  $job_sub .= q{'};

  if ( $self->verbose() ) {
    $self->log($job_sub);
  }

  return $job_sub;
}

sub _check_bam_cluster_count {
   my $self = shift;

   $self->log('Checking cluster count in bam files based on flagstats');
   my  $expected_cluster_count = $self->_bustard_pf_cluster_count();
   $self->log(qq{Expected cluster count: $expected_cluster_count});

   my $total_bam_cluster_count;

   if ( $self->is_multiplexed_lane($self->position() ) ) {
      $total_bam_cluster_count += $self->_bam_cluster_count_total({plex=>1});
   }else{
      $total_bam_cluster_count += $self->_bam_cluster_count_total({});
   }

   if($self->is_paired_read()){
       $total_bam_cluster_count /= 2;
    }

    $self->log(qq{Actual cluster count in bam files: $total_bam_cluster_count});

    if($expected_cluster_count != $total_bam_cluster_count ){
        my $msg = qq{Cluster count in bam files not as expected\n\tExpected: $expected_cluster_count\n\tActual:$total_bam_cluster_count };
        $self->log($msg);
        croak $msg;
    }
    $self->log('Bam files have correct cluster count');

    return 1;
}

sub _bam_cluster_count_total {
   my ( $self, $args_ref ) = @_;

   my $plex = $args_ref->{plex};

   my $bam_cluster_count = 0;

   my $qc_store = npg_qc::autoqc::qc_store->new( use_db => q[no] );

   my $archive_path = $self->archive_path();
   my $position = $self->position();

   my $qc_path_to_load = $archive_path;
   if( $plex ){
      $qc_path_to_load .= q{/lane}.$position;
   }
   $qc_path_to_load .= q{/qc};

   my $collection = $qc_store->load_from_path( $qc_path_to_load );

   if( !$collection || $collection->is_empty() ){
     $self->log("There is no auto qc results available here: $qc_path_to_load");
     return $bam_cluster_count;
   }

   my $collection_lane = $collection->slice(q[position], $position);
   my $bam_flagstats_collection = $collection_lane->slice('class_name', 'bam_flagstats');

   if( !$bam_flagstats_collection || $bam_flagstats_collection->is_empty() ){
     $self->log("There is no bam flagstats available for this lane $position in here: $qc_path_to_load");
     return $bam_cluster_count;
   }

   my $bam_flagstats_objs = $bam_flagstats_collection->results();

   foreach my $bam_flagstats (@{$bam_flagstats_objs}){

      if( $bam_flagstats->id_run() != $self->id_run() ){
         next;
      }

      $bam_cluster_count += $bam_flagstats->total_reads();
   }

   return $bam_cluster_count;
}

no Moose;

__PACKAGE__->meta->make_immutable;

1;
__END__

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Moose

=item Carp

=item English -no_match_vars

=item Readonly

=item File::Spec

=item npg_pipeline::archive

=item List::MoreUtils

=item XML::LibXML

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

$Author: mg8 $

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2011 GRL, by Guoying Qi (gq1@sanger.ac.uk)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
