#############
# $Id: cram.pm 15874 2012-07-12 09:19:16Z kl2 $
# Created By: kl2
# Last Maintained By: $Author: kl2 $
# Created On: 2012-06-11
# Last Changed On: $Date: 2012-07-12 10:19:16 +0100 (Thu, 12 Jul 2012) $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-pipeline/tags/release-39.0/lib/npg_pipeline/archive/file/generation/cram.pm $

package npg_pipeline::archive::file::generation::cram;
use Moose;
use Carp;
use File::Basename;
use English qw{-no_match_vars};
use Readonly;
use List::MoreUtils qw{any};

use st::api::lims;
use npg_common::sequence::reference;

extends q{npg_pipeline::archive::file::generation};

with qw{
  npg_common::roles::run::lane::tag_info
};

Readonly::Scalar our $VERSION  => do { my ($r) = q$LastChangedRevision: 15874 $ =~ /(\d+)/mxs; $r; };

# these have been found to be the best for bam generation, so have been hard coded. May need to be reconsidered for cram generation
Readonly::Scalar our $DEFAULT_NUMBER_OF_PROCESSORS => 4;

=head2 generate

main method to get a list of lanes and (optionally) plexes for a given run and 
  create bsub submissions to cram them

=cut

sub generate {
    my ($self, $arg_refs) = @_;

    #####################################################################
    # Use st::api::lims to determine lane and plex setup for the run.
    # Create a hash with lane numbers as keys and array ref values. The
    # array ref contains tag_index values for the lane and will be empty
    #  if the lane isn't plexed - this determines how the jobs arrays are
    # contructed in _generate_bsub_commands()
    #
    # The script invoked will be responsible for determining which bam
    # files are actually present at that level, checking the files in the
    # relevant run_folder - i.e human[_split], phix)
    #####################################################################
    my @bsub_commands;
    my @job_ids;
    my %lane_tagindexes = ();
    my $id_run = $self->id_run;

    $self->log( qq{Creating Jobs to create cram files for $id_run} );
    my $run_lims = st::api::lims->new(id_run => $id_run);
    for my $lane_lims ($run_lims->associated_child_lims) {
        if($lane_lims->is_pool) {
            for my $tag_lims ($lane_lims->associated_child_lims) {
                push @{$lane_tagindexes{$lane_lims->{position}}}, $tag_lims->{tag_index};
                $self->log( qq{Pushed lane/tag_index $lane_lims->{position}/$tag_lims->{tag_index} into plex array} );
            }
        }
        else {  # not plexed, just put this run/lane into queue
            $lane_tagindexes{$lane_lims->{position}} = [];
            $self->log( qq{Empty tag array for lane $lane_lims->{position} (lane level processing)} );
        }
    }

    # add tag zero for plexed lanes
    for my $lane (keys %lane_tagindexes) {
	if(@{$lane_tagindexes{$lane}} > 0) {
            push @{$lane_tagindexes{$lane}}, 0;
            $self->log( qq{Pushed tag 0 for lane $lane} );
        }
    }

    push @bsub_commands, $self->_generate_bsub_commands(\%lane_tagindexes, $arg_refs->{required_job_completion});

    for my $cmd (@bsub_commands) {
        push @job_ids, $self->submit_bsub_command($cmd);
    }

    return @job_ids;  # or does this cause an unwanted dependency on these lsf jobs?
}

#############################################################
# responsible for generating the bsub commands to be executed
#  grouped into lanes with plexes and those without
#############################################################
sub _generate_bsub_commands {
    my ($self, $lanes_tagindexes, $required_job_completion) = @_;

    my $id_run = $self->id_run;
    my @non_multi_lanes = ();
    my $bam_basename;

    $self->clear_lane_tag_pairs();
    for my $lane (keys %{$lanes_tagindexes}) {
        if($lanes_tagindexes->{$lane} and @{$lanes_tagindexes->{$lane}} > 0) { # array ref of tag indexes present (multiplexed)?
            for my $tag_index (@{$lanes_tagindexes->{$lane}}) {  # set up for create_array_string() call below
                $self->add_lane_tag_pair( { position => $lane, tag_index => $tag_index } );
            }
        }
        else {
            push @non_multi_lanes, $lane;
        }
    }

    # some values in common for multiplexed and non-multiplexed lanes
    my @bsub_commands;
    my $resources = ($self->fs_resource_string({ counter_slots_per_job => $DEFAULT_NUMBER_OF_PROCESSORS, resource_string => $self->_default_resources(), }));
    $resources ||= q{};
    $required_job_completion ||= q{};
    my $script_bin = $self->internal_script_bin;
    my $archive_path = $self->archive_path();
    $bam_basename = $id_run . q{_};

    if(@non_multi_lanes) {    # non-multiplexed lanes
        my $lsf_array_string = $self->create_array_string(@non_multi_lanes);
        ## no critic (ValuesAndExpressions::RequireInterpolationOfMetachars)
        my $job_command = qq{$script_bin/} . $self->external_script_names_conf()->{cram_creator_script} . qq{ --id_run $id_run } . q{ --position $LSB_JOBINDEX} . q{ --archive_path } . $archive_path;
        ## use critic
        $self->log( qq{Lane level - generated job_command: $job_command} );

        # now the bsub parameters specific to non-multiplexed lanes
        my $job_name = $self->external_script_names_conf()->{cram_creator_script} . q{_} . $self->id_run . q{_} . $self->timestamp();
        my $loop_job_name = $self->_get_loop_job_name($job_name, $lsf_array_string, 0);
        $self->log( qq{Lane level - generated loop_job_name: $loop_job_name} );
        my $loop_outfile = $self->_get_loop_outfile($job_name, 0);

        my $job_sub =  q{bsub -q } . $self->lsf_queue() . qq{ $resources $required_job_completion -J $loop_job_name -o $loop_outfile '$job_command'};

        push @bsub_commands, $job_sub;   # end of job submission creation
    }

    if($self->count_lane_tag_pairs > 0) {    # multiplexed lanes
        my $lsf_array_string = $self->lsf_job_array_from_lane_tag_pairs;

        my $position_string = $self->position_decode_string;
        my $tag_index_string = $self->tag_index_decode_string;
        my $job_command = qq{$script_bin/} . $self->external_script_names_conf()->{cram_creator_script} . qq{ --id_run $id_run } . qq{ --position $position_string } . qq{ --tag_index $tag_index_string} . q{ --archive_path } . $archive_path;
        $self->log( qq{Plexed - generated job_command: $job_command} );

        # now the bsub parameters specific to multiplexed lanes
        my $job_name = $self->external_script_names_conf()->{cram_creator_script} . q{_} . $self->id_run . q{_} . $self->timestamp();
        my $loop_job_name = $self->_get_loop_job_name($job_name, $lsf_array_string, 1);
        $self->log( qq{Plexed - generated loop_job_name: $loop_job_name} );
        my $loop_outfile = $self->_get_loop_outfile($job_name, 1);

        my $job_sub =  q{bsub -q } . $self->lsf_queue() . qq{ $resources $required_job_completion -J $loop_job_name -o $loop_outfile '$job_command'};

        push @bsub_commands, $job_sub;   # end of job submission creation
    }

    return @bsub_commands;
}

sub _default_resources {
    my ($self) = @_;

    my $mem = $self->general_values_conf()->{bam_creation_memory};

    return qq{-R 'select[mem>$mem] rusage[mem=$mem] span[hosts=1]' -M${mem}000 -n} . $DEFAULT_NUMBER_OF_PROCESSORS;
}

sub _get_loop_job_name {
    my ($self, $job_name, $lsf_array_string, $multiplex) = @_;
    my $id_run = $self->id_run;

    $job_name .= $lsf_array_string;
    if(! $self->no_array_cpu_limit()) {
        $job_name .= q{%} . $self->array_cpu_limit;
    }
    $job_name = q{'} . $job_name . q{'};
    if($multiplex) {
        $job_name =~ s/_{$id_run}_/_{$id_run}_mp_/xms;
    }

    return $job_name;
}

sub _get_loop_outfile {
    my ($self, $job_name, $multiplex) = @_;

    my $outfile = $self->make_log_dir($self->archive_path) . q{/} . $self->bsub_output_file;
    my $out_subs_name = $job_name . q{.%I.%J};
    $outfile =~ s/substitute_this/$out_subs_name/xms;
    if($multiplex) {
        my $id_run = $self->id_run;
        $outfile  =~ s/_{$id_run}_/_{$id_run}_mp_/xms;
    }

    return $outfile;
}

no Moose;
__PACKAGE__->meta->make_immutable;
1;
__END__

=head1 NAME

npg_pipeline::archive::file::generation::cram

=head1 VERSION

$LastChangedRevision: 15874 $

=head1 SYNOPSIS

  my $oAfgfq = npg_pipeline::archive::file::generation::cram->new({
    run_folder => $sRunFolder,
  });

=head1 DESCRIPTION

Object module which knows how to construct and submits the command line to LSF for creating cram files.

=head1 SUBROUTINES/METHODS

=head2 generate - generates the bsub jobs and submits them for creating the cram files, returning an array of job_ids.

  my @job_ids = $oAfgfq->generate({
    required_job_completion} => q{-w (123 && 321)};
    timestamp                => q{20090709-123456};
    solexa_bin               => q{/software/solexa/bin};
    id_run                   => 1234;
    BSUB_ERROR_FILE          => q{substitute_this.err};
    BSUB_OUTPUT_FILE         => q{substitute_this.out};
  });

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Carp

=item English -no_match_vars

=item Readonly

=item Moose

=item npg_pipeline::archive::file::generation

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

$Author: kl2 $

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2012 GRL, by Kevin Lewis (kl2@sanger.ac.uk)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
