#############
# $Id: harold_analysis_runner.pm 16184 2012-11-04 10:05:28Z dj3 $
# Created By: ajb
# Last Maintained By: $Author: dj3 $
# Created On: 2009-09-14
# Last Changed On: $Date: 2012-11-04 10:05:28 +0000 (Sun, 04 Nov 2012) $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-pipeline/tags/release-39.0/lib/npg_pipeline/daemons/harold_analysis_runner.pm $

package npg_pipeline::daemons::harold_analysis_runner;
use Moose;
use Carp;
use English qw{-no_match_vars};
use Readonly; Readonly::Scalar our $VERSION => do { my ($r) = q$LastChangedRevision: 16184 $ =~ /(\d+)/mxs; $r; };
use npg::api::run_status_dict;
use st::api::lims;

extends qw{npg_pipeline::daemons};

Readonly::Scalar our $PB_CAL_SCRIPT     => q{npg_pipeline_PB_cal_bam};

Readonly::Scalar our $PB_CAL_OPTIONS  => q{};
Readonly::Scalar our $DEFAULT_JOB_PRIORITY       => 50;
Readonly::Scalar our $DEFAULT_MISEQ_JOB_PRIORITY => 60;
Readonly::Scalar our $ANALYSIS_PENDING  => q{analysis pending};

sub run {
  my ($self) = @_;

  #################
  # go through each run, running the npg_pipeline_analyse_RTA script on each
  # run on first instance of seeing an end of the run, and skipping the other end
  $self->log(q{Analysis daemon running...});

  foreach my $run ( @{ npg::api::run_status_dict->new({description => $ANALYSIS_PENDING,})->runs() } ) {

     eval{
        $self->_process_one_run($run);
        1;
     } or do {
        $self->log('Problems to process one run ' . $run->id_run() );
        $self->log($EVAL_ERROR);
        next;
     }

  }

  return 1;
}

##############
# Private methods

sub _process_one_run {
    my ($self, $run) = @_;

    my $id_run = $run->id_run();
    # skip if you have already seen this
    if ($self->_seen->{$id_run}) {
      $self->log(qq{Already seen $id_run});
      return;
    }

    my $batch_id = $run->batch_id();
    if(!$batch_id){
       $self->log(qq{NO BATCH ID for run $id_run});
       return;
    }

    my $inst_model = $run->instrument->model();

    # only run if this run is an rta run
    if ( $run->is_rta() ) {

      my $run_is_spiked;
      my $run_has_control;
      my @lane_lims = st::api::lims->new(batch_id => $batch_id)->associated_child_lims;
      my $lanes = [];
      foreach my $lane ( @lane_lims ) {
        if ( $lane->spiked_phix_tag_index ) {
          $run_is_spiked++;
        }
        if ( $lane->is_control ) {
          $run_has_control++;
        }
        push @{ $lanes }, $lane->position;
      }

      my $arg_refs = {
        id_run => $id_run,
        script => $PB_CAL_SCRIPT,
        lanes => $lanes,
      };
      if (my $job_priority = $run->priority) {
        $arg_refs->{job_priority} = $job_priority +
          ( $inst_model eq q{MiSeq} ? $DEFAULT_MISEQ_JOB_PRIORITY : $DEFAULT_JOB_PRIORITY );
      }

      $arg_refs->{options} = $run_is_spiked ? $PB_CAL_OPTIONS : $PB_CAL_OPTIONS . q{ --no_recalibration};

      eval {
        $self->_run_command( $id_run, $self->_generate_command( $arg_refs ) );
      } or do {
        $self->log( $EVAL_ERROR );
        return;
      };

    } else {
      $self->log(qq{$id_run is not an rta run});
    }

    $self->_seen()->{$id_run}++; # you have now seen this
    return;
}

# generate the command to run
sub _generate_command {
  my ( $self, $arg_refs ) = @_;

  my $id_run  = $arg_refs->{id_run};
  my $script  = $arg_refs->{script};
  my $options = $arg_refs->{options};
  my $lanes = $arg_refs->{lanes} || [];
  my $job_priority = $arg_refs->{job_priority};

  my $cmd = 'source /software/lsf/conf/profile.lsf; ';  # Yes, we need this
  $cmd   .= $self->internal_script_bin() . q{/} . $script .
            ( $job_priority ? qq{ --job_priority $job_priority} : q() ) .
            q{ --verbose --id_run } . $id_run . $options;
  foreach my $lane ( @{ $lanes } ) {
    $cmd .= qq{ --lanes $lane};
  }
  return $cmd;
}

# run the command generated
sub _run_command {
  my ( $self, $id_run, $cmd ) = @_;

  $self->log(qq{Running $cmd});

  my $output = `$cmd`;

  if ( $CHILD_ERROR ) {
    croak qq{Error $CHILD_ERROR occured. Will try $id_run again on next loop.}
  };

  $self->log(qq{Output:\n$output});

  return 1;
}

no Moose;
__PACKAGE__->meta->make_immutable;
1;
__END__

=head1 NAME

npg_pipeline::daemons::harold_analysis_runner

=head1 VERSION

$LastChangedRevision: 16184 $

=head1 SYNOPSIS

  my $runner = npg_pipeline::daemons::harold_analysis_runner->new();
  $runner->run();

=head1 DESCRIPTION

This module interrogates the npg database for runs with a status of analysis pending, and that are rta runs, and then runs the npg_pipeline_analyse_RTA script on each of them

=head1 SUBROUTINES/METHODS

=head2 run - the only method and the only one you need. It does everything.

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Moose

=item Carp

=item English -no_match_vars

=item Readonly

=item npg_pipeline::daemons

=item npg::api::run_status_dics

=item st::api::lims

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

$Author: dj3 $

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by Andy Brown (ajb@sanger.ac.uk)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
