#############
# $Id: harold.pm 16101 2012-10-17 09:37:15Z mg8 $
# Created By: ajb
# Last Maintained By: $Author: mg8 $
# Created On: 2009-06-11
# Last Changed On: $Date: 2012-10-17 10:37:15 +0100 (Wed, 17 Oct 2012) $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-pipeline/tags/release-39.0/lib/npg_pipeline/pluggable/harold.pm $

package npg_pipeline::pluggable::harold;

use Moose;
use Carp;
use English qw{-no_match_vars};
use Readonly; Readonly::Scalar our $VERSION => do { my ($r) = q$LastChangedRevision: 16101 $ =~ /(\d+)/mxs; $r; };

use npg::api::request;
use npg_pipeline::archive::folder::WebCache;
use npg_common::irods::run::Interop;

extends q{npg_pipeline::pluggable};

with qw{npg_common::roles::run::long_info};
with q{npg_common::roles::run::status};

=head1 NAME

npg_pipeline::pluggable::harold

=head1 VERSION

$LastChangedRevision: 16101 $

=head1 SYNOPSIS

  my $harold = npg_pipeline::pluggable::harold->new(
      id_run => 1234,
      run_folder => q{123456_IL2_1234},
    );

=head1 DESCRIPTION

A collection of common pipeline functions.

=cut

Readonly::Array our @CHANGE_STATUS_FUNCTIONS => qw/set_run_status_analysis_complete
                                                   set_run_status_qc_complete
                                                   set_run_status_qc_review_pending
                                                   set_run_status_run_archived
                                                   set_run_status_archival_in_progress
                                                   set_run_status_analysis_in_progress
                                                   set_run_status_secondary_analysis_in_progress/;

Readonly::Array our @AUTOQC_FUNCTIONS => qw/ bam2fastqcheck_and_cached_fastq
                                             qc_cache_reads 
                                             qc_genotype 
                                             qc_qX_yield
                                             qc_insert_size
                                             qc_adapter
                                             qc_contamination 
                                             qc_ref_match 
                                             qc_sequence_error
                                             qc_gc_fraction 
                                             qc_gc_bias
                                             qc_pulldown_metrics
                                             qc_tag_metrics
                                           /;

=head1 SUBROUTINES/METHODS

=head2 BUILD

Called on new construction to ensure that certain parameters are filled/set up

=cut

sub BUILD {
  my ($self) = @_;

  if ( !$self->has_id_run() && !$self->has_run_folder() ) {
    croak q{Error: No run_folder or id_run provided};
  }

  if (!$self->has_log_file_path()) {
    $self->set_log_file_path( $self->runfolder_path() );
  }

  if (!$self->has_log_file_name()) {
    my $log_name = $PROGRAM_NAME . q{_} . $self->id_run();
    if ( ! $self->no_lanes() || $self->lane() ) {
      foreach my $lane ( $self->positions() ) {
        $log_name .= qq{_$lane};
      }
    }
    $log_name .= q{_} . $self->timestamp() . q{.log};
    $log_name =~ s{/}{_}gmxs; # if $PROGRAM_NAME includes a directory path, then in log_name, change / to _
    $log_name =~ s{\A_}{}xms;
    $self->set_log_file_name($log_name);
  };

  # for RTA runs, the intensities directory is generated on the instrument. If it hasn't been
  # already given, then set this up for all analysis
  if ( !$self->has_intensity_path() ) {
    $self->_set_intensity_path( $self->runfolder_path() . q{/Data/Intensities} );
  }

  $self->_inject_status_change_functions();
  $self->_inject_autoqc_functions();
  return 1;
}

=head2 set_run_status_analysis_complete
=head2 set_run_status_qc_complete
=head2 set_run_status_qc_review_pending
=head2 set_run_status_run_archived
=head2 set_run_status_archival_in_progress
=head2 set_run_status_analysis_in_progress
=head2 set_run_status_secondary_analysis_in_progress

functions to update the run statuses to various statuses.

=cut
sub _inject_status_change_functions {

  foreach my $function (@CHANGE_STATUS_FUNCTIONS) {
    my $status = $function;
    $status =~ s/set_run_//sm;
    $status = uc $status;
    ##no critic (TestingAndDebugging::ProhibitNoStrict TestingAndDebugging::ProhibitNoWarnings)
    no strict 'refs';
    no warnings 'redefine';
    *{$function}= sub{  my ($self, @args) = @_; return $self->_update_run_status( $status, @args ); };
  }
  return;
}

=head2 bam2fastqcheck_and_cached_fastq
=head2 qc_cache_reads 
=head2 qc_genotype 
=head2 qc_qX_yield
=head2 qc_insert_size
=head2 qc_adapter
=head2 qc_contamination 
=head2 qc_ref_match 
=head2 qc_sequence_error
=head2 qc_gc_fraction 
=head2 qc_gc_bias
=head2 qc_tag_metrics

functions to run various autoqc checks.

=cut
sub _inject_autoqc_functions {

  foreach my $function (@AUTOQC_FUNCTIONS) {
    my $qc = $function;
    $qc =~ s/qc_//sm;
    if ($qc eq 'cache_reads') { $qc = 'cache'; }
    ##no critic (TestingAndDebugging::ProhibitNoStrict TestingAndDebugging::ProhibitNoWarnings)
    no strict 'refs';
    no warnings 'redefine';
    *{$function}= sub{  my ($self, @args) = @_;  return $self->_qc_runner($qc, @args); };
  }
  return;
}

sub _update_run_status {
  my ($self, $status, @args) = @_;

  # check that this hasn't been expressly turned off
  if ($self->no_status_updates()) {
    $self->log(q{Update status turned off});
    return ();
  }

  $status = $self->run_statuses()->{ $status };
  my $required_job_completion = shift @args;

  my $rsu = $self->new_with_cloned_attributes(q{npg_pipeline::run::status::update},{
    status_one => $status,
    status_two => $status,
  });
  my $arg_refs = {
    required_job_completion => $required_job_completion,
  };

  if (!$required_job_completion) {
    $rsu->update($arg_refs);
    return ();
  }

  my $job_id = $rsu->submit_update($arg_refs);
  return ($job_id);
}

sub _update_run_lane_status {
  my ( $self, $status, @args ) = @_;

  if ($self->no_status_updates()) {
    $self->log(q{Update status turned off});
    return ();
  }

  my $required_job_completion = shift @args;

  my $schema = $self->npg_tracking_schema();
  if (!$schema) {
    croak q{Unable to obtain dbix tracking schema};
  }

  if (!$required_job_completion) {
    foreach my $lane ( $self->positions() ) {
      require npg_pipeline::run::status::UpdateLane;
      npg_pipeline::run::status::UpdateLane->new( {
        status => $status,
        position => $lane,
        id_run => $self->id_run(),
        npg_tracking_schema => $schema,
      } )->update();
    }
    return ();
  }

  my $rlsu = $self->new_with_cloned_attributes(q{npg_pipeline::run::status::UpdateLane},{
    status => $status,
  });
  my $arg_refs = {
    required_job_completion => $required_job_completion,
  };

  return $rlsu->launch_bsub_jobs($arg_refs);
}

################
# single method to actually run qc, which should be given the test from the called method requested
sub _qc_runner {
  my ($self, $qc_to_run, @args) = @_;

  my $required_job_completion = shift @args;
  my $arg_refs = {
    required_job_completion => $required_job_completion,
  };

  if ($qc_to_run eq q{sequence_error4spiked_phix}) {
    $qc_to_run = q{sequence_error};
    $arg_refs->{spiked_phix_sequence} = q[--species PhiX --sequence_type phix];
  }

  my @job_ids = $self->new_with_cloned_attributes(q{npg_pipeline::archive::file::qc},{
    qc_to_run  => $qc_to_run,
    schema     => ref $self,
  })->run_qc($arg_refs);

  return @job_ids;
}

=head2 update_run_lane_analysis_in_progress
=head2 update_run_lane_analysis_complete

Functions to update a run lane to various statuses

=cut

sub update_run_lane_analysis_in_progress {
  my ( $self, @args ) = @_;
  return $self->_update_run_lane_status( q{analysis in progress}, @args);
}

sub update_run_lane_analysis_complete {
  my ( $self, @args ) = @_;
  return $self->_update_run_lane_status( q{analysis complete}, @args);
}

=head2 spider

Function which will go out and spider over webservices to cache on the filesystem all data from npg and sequencescape
This should be run up front of any jobs being submitted, and will not submit any jobs to lsf
If the environment variable is already set which is to be set for spidering, it will assume that it has already been
done, so post_qseq can have it as a default to run, but not redo if launched from analyse_RTA

=cut

sub spider {
  my ( $self ) = @_;

  my $cache_dir_var_name = npg::api::request->cache_dir_var_name();
  # if the environment variable is already set, then there is no need to spider out
  if ( $ENV{ $cache_dir_var_name } ) {
    $self->log( q[$]. q[ENV{] . $cache_dir_var_name . q[} already set to ] . $ENV{$cache_dir_var_name} . q{, not spidering!} );
    return ();
  }

  my $cache_dir = $self->new_with_cloned_attributes( q{npg_pipeline::archive::folder::WebCache} )->webcache_directory_link();
  if ($cache_dir) {
    $self->log( q[Setting $] . q[ENV{] . $cache_dir_var_name . q[} to linked ] . $cache_dir );
    $ENV{$cache_dir_var_name} = $cache_dir; ## no critic (Variables::RequireLocalizedPunctuationVars)
    return ();
  }
  if ($self->no_spider()) {
    $self->log( q{Spidering turned off} );
    return ();
  }

  $cache_dir = $self->runfolder_path() . q{/cache_webservice_analysis_} . $self->timestamp();
  qx{mkdir $cache_dir};
  if ( $CHILD_ERROR ) {
    croak q{problem creating cache directory: } . $cache_dir;
  }

  $self->log( q[Setting $] . q[ENV{] . $cache_dir_var_name . q[} to ] . $cache_dir );
  $ENV{$cache_dir_var_name} = $cache_dir; ## no critic (Variables::RequireLocalizedPunctuationVars)
  local $ENV{npg::api::request->save2cache_dir_var_name()} = 1;
  $self->spider_runner();
  local $ENV{npg::api::request->save2cache_dir_var_name()} = 0;

  return ();
}

=head2 fix_config_files

check config files have correct data, and fix runfolder,instrument,id_run
this will not be a job launched on the farm, and so should be taken into account
i.e. don't expect it to run after something has been submitted, expecting a dependency
=cut

sub fix_config_files {
  my ( $self, @args ) = @_;

  if ( ! $self->no_fix_config_files() ) {
    $self->new_with_cloned_attributes( q{npg_pipeline::analysis::FixConfigFiles} )->run();
  }

  return ();
}

=head2 illumina2bam

=cut

sub illumina2bam {
   my ( $self, @args ) = @_;

   my $illumina2bam = $self->new_with_cloned_attributes(q{npg_pipeline::archive::file::generation::illumina2bam});

   my $required_job_completion = shift @args;

   my @job_ids;
   eval {
     @job_ids = $illumina2bam->generate({required_job_completion => $required_job_completion,});
     1;
   } or do {
      croak q{Problem with illumina2bam jobs: } . $EVAL_ERROR;
   };

   return @job_ids;
}

=head2 touch_completed_lane

launches a job which will touch a file in the archive directory to say that post_qseq has completed

=cut

sub touch_completed_lane {
  my ( $self, @args ) = @_;

  my $arg_refs = {
    required_job_completion => shift @args,
  };
  return $self->new_with_cloned_attributes( q{npg_pipeline::archive::file::generation::touchCompletedLane} )->launch_bsub_job($arg_refs);
}

=head2 create_webcache_softlink

create a relative softlink to the webcache directory used for this analysis in the archive directory

=cut

sub create_webcache_softlink {
  my ( $self, @args ) = @_;

  return $self->new_with_cloned_attributes( q{npg_pipeline::archive::folder::WebCache} )->create_webcache_link( @args );
}

=head2 create_summary_link_analysis

function which creates/changes the summary link in the runfolder

=cut

sub create_summary_link_analysis {
  my ($self, @args) = @_;

  # check that this hasn't been expressly turned off
  if ($self->no_summary_link()) {
    $self->log(q{Summary link creation turned off});
    return ();
  }

  my $required_job_completion = shift @args;

  my $rfl = $self->new_with_cloned_attributes(q{npg_pipeline::run::folder::link},{
    analysis_type => q{full},
    folder        => q{analysis},
  });

  my $arg_refs = {
    required_job_completion => $required_job_completion,
  };

  if (!$required_job_completion) {
    $rfl->make_link($arg_refs);
    return ();
  }

  my $job_id = $rfl->submit_create_link($arg_refs);
  return ($job_id);
}

=head2 move_to_outgoing

used in archival step (post qc review) for moving the runfolder to the outgoing directory

=cut

sub move_to_outgoing {
  my ($self, @args) = @_;

  # check that this hasn't been expressly turned off
  if ($self->no_folder_moves()) {
    $self->log(q{Folder moving turned off});
    return ();
  }

  my $required_job_completion = shift @args;
  my $rfm = $self->new_with_cloned_attributes(q{npg_pipeline::run::folder::move},{
    folder => q{outgoing},
  });

  my $arg_refs = {
    required_job_completion => $required_job_completion,
  };

  if (!$required_job_completion) {
    $rfm->move_runfolder($arg_refs);
    return ();
  }

  my $job_id = $rfm->submit_move_run_folder($arg_refs);
  return ($job_id);

}

=head2 create_empty_fastq

Creates a full set of empty fastq files

=cut
sub create_empty_fastq {
  my ( $self, @args ) = @_;
  return $self->new_with_cloned_attributes(
      q{npg_pipeline::archive::file::generation})->create_empty_fastq_files();
}

no Moose;
1;
__END__

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Moose

=item Carp

=item English -no_match_vars

=item Readonly

=item npg::api::request

=item npg_common::roles::run::long_info

=item npg_common::roles::run::status

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

$Author: mg8 $

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by Andy Brown (ajb@sanger.ac.uk)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
