#############
# $Id: PB_cal_bam.pm 16492 2013-01-15 17:16:26Z ib5 $
# Created By: gq1
# Last Maintained By: $Author: ib5 $
# Created On: 2010-08-01
# Last Changed On: $Date: 2013-01-15 17:16:26 +0000 (Tue, 15 Jan 2013) $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-pipeline/tags/release-39.0/lib/npg_pipeline/pluggable/harold/PB_cal_bam.pm $

package npg_pipeline::pluggable::harold::PB_cal_bam;
use Moose;
use Carp;
use English qw{-no_match_vars};
use Readonly; Readonly::Scalar our $VERSION => do { my ($r) = q$LastChangedRevision: 16492 $ =~ /(\d+)/mxs; $r; };

extends qw{npg_pipeline::pluggable::harold::post_qc_review};

=head1 NAME

npg_pipeline::pluggable::harold::PB_cal_bam

=head1 VERSION

$LastChangedRevision: 16492 $

=head1 SYNOPSIS

  npg_pipeline::pluggable::harold::PB_cal_bam->new(id_run => 333)->main();

=head1 DESCRIPTION

Pluggable module runner for the PB_cal_bam pipeline

=cut

Readonly::Array our @OLB_FUNCTIONS => qw/ matrix_lanes    matrix_all
                                          phasing_lanes   phasing_all
                                          basecalls_lanes basecalls_all
                                        /;
=head1 SUBROUTINES/METHODS

=head2 BUILD

ensures that the folders are built that everything expects. If they are provided, it will use them, else it will create them

Notes:

1) intensity_path - there is a bug in the imported path_info role which needs fixing. As such, at this time, if intensity_path isn't
directly specified, then it will take runfolder_path and add /Data/Intensities

2) if recalibrated_path, pb_cal_path, bustard_path and basecall_path are specified, it will use these (and map pb_cal_path <->
recalibrated_path and bustard_path <-> basecall_path ), else it will create based on whether no_recalibration is set, and utilise
the timestamp

3) This is designed to be launched after the harold.pm BUILD method is called as per Moose v1.15. If this should be altered in
a future version of Moose, then this may break

=cut

sub BUILD {
  my ( $self ) = @_;

  if ( ! $self->has_intensity_path() ) {
    $self->_set_intensity_path( $self->runfolder_path() . q{/Data/Intensities} );
  }
  $self->log('Intensities path: ' . $self->intensity_path() );

  # If preprocessing with OLB, to set the paths mentioned below,
  # one needs to know the name of the bustard directory.
  # This name is not known till the bustard scripts is run.
  #Therefore, if using OLB, delay creating these directories.
  if (!$self->olb) {
    if ( ! $self->has_dif_files_path() ) {
      $self->_set_dif_files_path( $self->intensity_path() );
    }
    $self->log('Dif files path: ' . $self->dif_files_path() );

    if ( ! $self->has_basecall_path() ) {
      $self->_set_basecall_path( $self->intensity_path() . q{/BaseCalls} );
    }
    $self->log('BaseCalls path: ' . $self->basecall_path() );

    my $timestamp = $self->timestamp();
    if( !  $self->has_bam_basecall_path() ){

      my $bam_basecalls_dir = $self->intensity_path() . q{/} .q{BAM_basecalls_} . $timestamp;
      $self->make_log_dir( $bam_basecalls_dir  );
      $self->_set_bam_basecall_path( $bam_basecalls_dir );
    }
    $self->log('BAM_basecall path: ' . $self->bam_basecall_path());
    $self->_set_bam_basecall_dependent_paths();
  }

  return 1;
}


=head2 _set_bam_basecall_dependant_paths

 If unset, sets recalibrated_path and pb_cal_path.

=cut
sub _set_bam_basecall_dependent_paths {
  my $self = shift;
  my $pathways = {
    recalibrated_path => undef,
    pb_cal_path => undef,
  };

  # for each of the paths, see if they have been prepopulated
  foreach my $path ( keys %{ $pathways } ) {
    my $has_method = q{has_} . $path;
    if ( $self->$has_method() ) {
      $pathways->{$path} = $self->$path();
    }
  }

  # if recalibrated_path or pb_cal_path are not set, but the other is, match them up
  if ( $pathways->{recalibrated_path} && ! $pathways->{pb_cal_path} ) {
    $pathways->{pb_cal_path} = $pathways->{recalibrated_path};
  }
  if ( ! $pathways->{recalibrated_path} && $pathways->{pb_cal_path} ) {
    $pathways->{recalibrated_path} = $pathways->{pb_cal_path};
  }

  # if there is no recalibrated_path and pb_cal_path, then create them and store
  if ( ! $pathways->{recalibrated_path} ) {
    my $recalibrated_level_dir = $self->no_recalibration() ? q{no_cal}
                               :                             q{PB_cal_bam}
                               ;
    $self->make_log_dir( $self->bam_basecall_path() . q{/} . $recalibrated_level_dir );
    $pathways->{recalibrated_path} = $self->bam_basecall_path() . q{/} . $recalibrated_level_dir;
    $pathways->{pb_cal_path}       = $self->bam_basecall_path() . q{/} . $recalibrated_level_dir;
  }
  # for each of these, go and set them (we know we must have created them by now)
  foreach my $path ( keys %{ $pathways } ) {
    my $set_method = q{_set_} . $path;
    $self->$set_method( $pathways->{$path} );
  }

  $self->log('PB_cal path: ' . $self->pb_cal_path());
  $self->log('Recalibrated_path: ' . $self->recalibrated_path() );
  return;
}

=head2 _inject_bustard_functions

 Dynamically creates functions to run OLB preprocessing.

=cut

sub _inject_bustard_functions {
  my $self = shift;

  foreach my $function (@OLB_FUNCTIONS) {
    ##no critic (TestingAndDebugging::ProhibitNoStrict)
    no strict 'refs';
    my $fpointer = 'bustard_' . $function;
    if ($self->olb) {
      *{$fpointer}= sub {  my ($self, @args) = @_;
                           my $job_dep = shift @args;
                           return npg_pipeline::analysis::bustard4pbcb->new(
                             pipeline=>$self,
                             bustard_home=>$self->intensity_path,
                             bustard_dir=>$self->bam_basecall_path,
                             id_run=>$self->id_run,
                             lanes=>$self->lanes)->make($function,$job_dep); };
    } else {
      *{$fpointer}= sub {  $self->log('OLB preprocessing switched off, not running ' . $function ); return (); }
    }
  }
  return;
}

=head2 prepare

 Actions to be performed before the functions can be run. Namely, if running OLB preprocessing,
 creates bustard directory and dependent directories.

=cut

sub prepare {
  my $self = shift;
  if ($self->olb) {
    my $bustard_dir = $self->new_with_cloned_attributes(q{npg_pipeline::analysis::bustard4pbcb},
                      {bustard_home => $self->intensity_path,})->bustard_dir();
    $self->_set_dif_files_path( $bustard_dir );
    $self->_set_basecall_path( $bustard_dir );
    $self->_set_bam_basecall_path( $bustard_dir );
    $self->log("basecall, bam_basecall and dif_files paths set to $bustard_dir");
    $self->make_log_dir( $bustard_dir  );
    $self->_set_bam_basecall_dependent_paths();
  }
  $self->_inject_bustard_functions();
  return;
}

=head2 make_multiplex_lane_specific_tag_files

This doesn't need to be submitted as a bsub job at this time.
It will create lane specific tag files, so that these can be used for index determination 
(unless a tag file has been provided, in which case, that will be used)

=cut

sub make_multiplex_lane_specific_tag_files {     # TODO: there's another copy of this in harold/bustard.pm
  my ($self, @args) = @_;
  $self->new_with_cloned_attributes(q{npg_pipeline::analysis::create_lane_tag_file})->generate();
  return ();
}

=head2 illumina_basecall_stats

Use Illumina tools to generate the (per run) BustardSummary and IVC reports (from on instrument RTA basecalling).

=cut

sub illumina_basecall_stats {
  my ($self, @args) = @_;
  if ($self->olb) {
    $self->log( q{OLB flag set, illumina_basecall_stats will not be run} );
    return ();
  }
  return $self->_run_harold_steps( q{generate_illumina_basecall_stats}, @args);
}

=head2 harold_alignment_files

Generate the alignment files to now be used for generating calibration tables

=cut

sub harold_alignment_files {
  my ($self, @args) = @_;

  return $self->_run_harold_steps( q{generate_alignment_files}, @args);
}

=head2 harold_calibration_tables

Generate the calibration tables used for harold recalibration

=cut

sub harold_calibration_tables {
  my ($self, @args) = @_;

  if ( $self->no_recalibration() ) {
    $self->log( q{no_recalibration declared, no recalibration will be performed} );
    return ();
  }
  return $self->_run_harold_steps( q{generate_calibration_table}, @args);
}

=head2 harold_recalibration

submit the recalibration jobs

=cut

sub harold_recalibration {
  my ($self, @args) = @_;
  return $self->_run_harold_steps( q{generate_recalibrated_bam}, @args);
}

sub _run_harold_steps {
  my ($self, $method, @args) = @_;
  my $required_job_completion = shift @args;
  my $harold = $self->new_with_cloned_attributes(q{npg_pipeline::analysis::harold_calibration_bam});
  return $harold->$method({required_job_completion => $required_job_completion,});
}

=head2 split_bam_by_tag

split lane bam file by indexing tag, marked by read group id

=cut

sub split_bam_by_tag {
  my ($self, @args) = @_;
  my $required_job_completion = shift @args;
  return $self->new_with_cloned_attributes( q{npg_pipeline::analysis::split_bam_by_tag} )
           ->generate({required_job_completion => $required_job_completion,});
}

=head2 create_archive_directory

creates the archive and qc directories,

and lane and lane qc directories if the lane is multiplexed.

=cut

sub create_archive_directory {
  my ($self, @args) = @_;
  $self->new_with_cloned_attributes(q{npg_pipeline::archive::folder::generation})->create_dir();
  return ();
}


=head2 bam_alignment

for each bam file for a plex or a lane(non-indexed lane), doing alignment, split spiked phix and non-consented data

=cut

sub bam_alignment {
  my ($self, @args) = @_;
  my $required_job_completion = shift @args;
  return $self->new_with_cloned_attributes(q{npg_pipeline::archive::file::generation::bam_alignment})
           ->generate({required_job_completion => $required_job_completion,});
}

=head2 bam_cluster_counter_check

For each lane, job submitted which checks that cluster counts are what they are expected to be in bam files

=cut

sub bam_cluster_counter_check {
  my ( $self, @args ) = @_;
  my $arg_refs = {required_job_completion => shift @args,};
  return $self->new_with_cloned_attributes( q{npg_pipeline::archive::file::BamClusterCounts} )->launch( $arg_refs );
}

=head2 cram_generation

For each lane, job submitted which checks that cluster counts are what they are expected to be in bam files

=cut

sub cram_generation {
    my ($self, @args) = @_;

    my $arg_refs = {required_job_completion => shift @args,};

    $self->new_with_cloned_attributes( q{npg_pipeline::archive::file::generation::cram} )->generate($arg_refs);

    return ();
}

=head2 tileviz

Flowcell surface visualisation

=cut

sub tileviz {
    my ($self, $required_job_completion) = @_;
    my ($bsub, $dir_out) = $self->_tileviz_bsub($required_job_completion);
    if ($bsub) {
        if (!-d $dir_out) {
	    $self->log(qq[Tileviz output directory $dir_out does not exist, creating...]);
	    mkdir $dir_out;
        }
        return $self->submit_bsub_command($bsub);
    } else {
        $self->log('Not running tileviz')
    }
    return ();
}

sub _tileviz_bsub {
    my ($self, $required_job_completion) = @_;

    my $lims = $self->lims->associated_child_lims_ia;
    my @lanes = ();
    foreach my $lane ($self->positions()) {
        my $l = $lims->{$lane};
        if ($l->spiked_phix_tag_index || $l->is_control) {
            push @lanes, $lane;
        }
    }

    if (@lanes) {
        @lanes = sort @lanes;
        my $qc_path = $self->qc_path;
        my $job_name = join q[_],  q{tileviz} , $self->id_run ,$self->timestamp();
        my $job_name_with_index = $job_name . '[' . join(q[,], @lanes) . ']';
        my $outfile = File::Spec->catfile($self->qc_path(), q[log], $job_name . q[.%I.%J.out]);
        my $nfs = $self->fs_resource_string({counter_slots_per_job => 2,});
        my $job_sub = join q[ ], 'bsub -q' , $self->lsf_queue(), '-m "sf6_1 sf5_2"', $nfs, $required_job_completion, '-J', $job_name_with_index, '-o', $outfile, $self->_and_preexecs($self->ref_adapter_pre_exec_string, "[ -d $qc_path ]");
        my $program = $self->pb_calibration_bin() . '/tileviz';
        my $file = $self->pb_cal_path() . q{/} . $self->id_run() . q{_} . $self->lsb_jobindex() . q{.bam};
        my $dir_out = File::Spec->catfile($qc_path, 'tileviz');
        my $snip_file = $self->control_snip_file();
		my $spatial_filter = $self->pb_calibration_bin() . '/spatial_filter';
#		my $num_tiles = srpipe::runfolder->new(id_run=>8077)->tile_count;
		my $num_tiles = $self->tile_count;
        $job_sub .= qq[ 'numactl --interleave=all -- $program --bam_in $file --dir_out $dir_out --snp_file $snip_file --mismatch_tool_path $spatial_filter --num_tiles $num_tiles'];
        return ($job_sub, $dir_out);
    }
    return ();
}

sub _and_preexecs {
  my ($self, $pe1, $pe2) = @_;
  $pe1 =~ s/'$//xms;
  return $pe1 . " && $pe2'";
}

no Moose;

__PACKAGE__->meta->make_immutable;
1;
__END__

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Moose

=item Carp

=item English -no_match_vars

=item Readonly

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

$Author: ib5 $

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2011 GRL, by Guoying Qi (gq1@sanger.ac.uk)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
