#############
# $Id: base.pm 16184 2012-11-04 10:05:28Z dj3 $
# Created By: ajb
# Last Maintained By: $Author: dj3 $
# Created On: 2009-11-03
# Last Changed On: $Date: 2012-11-04 10:05:28 +0000 (Sun, 04 Nov 2012) $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-pipeline/tags/release-39.0/lib/npg_pipeline/roles/business/base.pm $

package npg_pipeline::roles::business::base;

use strict;
use warnings;
use Moose::Role;
use Carp;
use Test::More;

use npg::api::run;
use st::api::lims;
use npg::api::run_status_dict;
use npg_qc::autoqc::checks::tag_metrics;

use Readonly; Readonly::Scalar our $VERSION => do { my ($r) = q$LastChangedRevision: 16184 $ =~ /(\d+)/mxs; $r; };

Readonly::Array my @POSITIONS => (1 .. 8);

=head1 NAME

npg_pipeline::roles::business::base

=head1 VERSION

$LastChangedRevision: 16184 $

=head1 SYNOPSIS

  package MyPackage;
  use Moose;
  ...
  with qw{npg_pipeline::roles::business::base};

=head1 DESCRIPTION

This role provides some base attributes and methods which are business logic related, but likely to be applied over the
whole of the pipeline. For information based items, where there is likely to need to be fixed information, use
npg_pipeline::roles::business::internal_info

=head1 SUBROUTINES/METHODS

=head2 run npg::api::run object, an id_run method is required for this.
=cut

has q{run} => (isa => q{npg::api::run},  is => q{ro}, metaclass => 'NoGetopt', lazy_build => 1);
sub _build_run {
  my ($self) = @_;
  return npg::api::run->new({id_run => $self->id_run(),});
}

=head2 lims st::api::lims batch-level object

=cut
has q{lims} => (isa => q{st::api::lims},  is => q{ro}, metaclass => 'NoGetopt', lazy_build => 1);
sub _build_lims {
  my ($self) = @_;
  return st::api::lims->new(id_run => $self->id_run);
}


=head2 positions

return an array of lane positions for this submission. If $arg_refs->{lane} or $self->lane() is provided, will return the contents of this as an array

  my @Positions = $oMySubClass->positions(); # will return the lanes on the run, or if unable (1,2,3,4,5,6,7,8)

  my @Positions = $oMySubClass->positions({
    lane => 5,
  }); # will return an array (5)

  my @Positions = $oMySubClass->positions({
    lane => [1,2,5,8],
  }); # will return an array (1,2,5,8)

=cut

sub positions {
  my ($self, $arg_refs) = @_;

  $arg_refs ||= {};

  if (!$arg_refs->{lane} && $self->has_lane()) {
    $arg_refs->{lane} = $self->lane();
  }

  my @positions = $arg_refs->{lane}   ? ($arg_refs->{lane})
                : !$self->no_lanes()  ? $self->all_lanes()
                : $self->all_positions();

  @positions = sort @positions;
  return @positions;
}


=head2 all_positions

return all lanes available in the flowcell, falling back on eight lanes

=cut

sub all_positions {
  my $self = shift;
  my @positions = $self->lims         ? map {$_->position()} $self->lims->associated_child_lims
                : $self->run          ? map {$_->position()} @{$self->run->run_lanes()}
                :                     @POSITIONS;
  @positions = sort @positions;
  return @positions;
}

=head2 processor_fork_number

integer containing the number of processors and forked jobs to be used in an lsf command, where it may need to be variable.

  my $iProcessorForkNumber = $class->processor_fork_number();

=cut

has q{processor_fork_number} => (isa => q{Int}, is => q{ro}, lazy_build => 1,
                      documentation => q{enables the user to set the number of processors and jobs to be forked},);
sub _build_processor_fork_number {
  my ($self) = @_;
  return $self->general_values_conf()->{default_nproc};
}

=head2 tile_list - a string of wildcards for tiles for OLB, defaults to an empty string

=cut

has q{tile_list} => (isa => q{Str}, is => q{ro}, default => q[],
                      documentation => q{string of wildcards for tiles for OLB, defaults to an empty string},);

=head2 override_all_bustard_options

Overrides all bustard options (including any given via other options) as a string - it is up to the user to ensure all are correct and given

=head2 has_override_all_bustard_options

predicate to ensure that options are available

=cut

has q{override_all_bustard_options} => (
  isa => q{Str},
  is => q{ro},
  predicate => q{has_override_all_bustard_options},
  documentation => q{Overrides all bustard options (including any given via other options) as a string - it is up to the user to ensure all are correct and given - i.e. only use if you know what you are doing.},
);

=head2 repository

for passing through a reference repository root directory, should you need to

=cut

has q{repository} => (
  isa       => q{Str},
  is        => q{ro},
  required  => 0,
  predicate => q{has_repository},
);

=head2 ref_repository

for passing through a reference repository, should you need to

=cut

has q{ref_repository} => (
  isa       => q{Str},
  is        => q{ro},
  required  => 0,
  metaclass => 'NoGetopt',
  lazy_build => 1,
);
sub _build_ref_repository {
  my ( $self ) = @_;
  return $self->repository() . q{/references};
}

=head2 control_ref

 Path to a default control reference for a default aligner

=cut

has q{control_ref} => (isa => q{Str}, is => q{ro}, lazy_build => 1,
                           documentation => q{path to a default control reference for a default aligner},);

sub _build_control_ref {
  my ( $self ) = @_;
  return $self->get_control_ref();
}

=head2 get_control_ref

 Path to a default control reference for an aligner given by the argument or, if no argument is given, fo a default aligner

=cut

sub get_control_ref {
  my ($self, $aligner) = @_;

  $aligner ||= $self->pb_cal_pipeline_conf()->{default_aligner};
  my $arg_refs = {
    aligner => $aligner,
    species => $self->general_values_conf()->{spiked_species},
  };
  if ( $self->repository() ) {
    $arg_refs->{repository} = $self->repository();
  }

  return Moose::Meta::Class->create_anon_class(
    roles => [qw/npg_common::sequence::reference::roles::find/])->new_object($arg_refs)->refs->[0];
}

=head2 control_snip_file

 Path to a default control reference snip file

=cut

sub control_snip_file {
  my $self = shift;

  my $path = $self->get_control_ref(q[snps]);
  if (!$path) {
    croak 'Failed to retrieve control snip file';
  }
  $path .= q[.rod];
  if (!-e $path) {
    croak "Snip file $path does not exists";
  }
  return $path;
}

=head2 analyse_rta_log_file

store and hold of analyse_rta_log_file

=head2 pb_cal_log_file

store and hold of pb_cal_log_file

=head2 post_qseq_rta_log_file

store and hold of post_qseq_log_file

=cut

has [ qw{
    analyse_rta_log_file
    pb_cal_log_file
    post_qseq_log_file
  } ] => (
  isa => q{Str},
  is  => q{ro},
  lazy_build => 1,
  builder => q{_build_any_log_file_name},
  documentation => q{only supply this if you know why you should},
);

sub _build_any_log_file_name {
  my ( $self ) = @_;
  my $log=  $self->log_file_path() . q{/} . $self->log_file_name();
  diag $log;
  return $log;
}

=head2 get_study_library_sample_names

given a position and a tag_index, return a hash with study, library and sample names 

=cut

sub get_study_library_sample_names {
  my ($self, $elims) = @_;

  my $sample_names = [];
  my %study_names_hash = ();

  my @alims = !$elims->is_pool ? ($elims) : $elims->associated_lims;
  foreach my $al (@alims) {

     my $sample_name = $al->sample_publishable_name();
     if($sample_name){
        push @{$sample_names}, $al->sample_publishable_name;
     }

     my $study_name = $al->study_publishable_name();
     my $study_description = $al->is_control ? npg_qc::autoqc::checks::tag_metrics->spiked_control_description : $al->study_description;
     if( $study_name ){
        if( $study_description ){
           $study_description =~ s/\n/\ /gmxs;
           $study_description =~ s/\t/\ /gmxs;
           $study_name .= q{: }.$study_description;
        }
        $study_names_hash{$study_name}++;
     }
  }

  my $library_aref = $elims->library_id ? [$elims->library_id] : [];
  my $href = {
          study    => [keys %study_names_hash],
          library  => $library_aref,
          sample   => $sample_names,
         };
  return $href;
}

=head2 spider_runner

Creates a cache of xml files from NPG and lims webservices

=cut

sub spider_runner {
  my $self = shift;

  $self->run->is_paired_run();
  $self->run->current_run_status();
  $self->run->instrument()->model();
  npg::api::run_status_dict->new()->run_status_dicts();
  my @methods = $self->lims->method_list();
  foreach my $l ( $self->lims->associated_lims() ) {
    foreach my $method ( @methods ) {
      $l->$method;
    }
  }
  return;
}

=head2 ref_adapter_pre_exec_string

pre-exec string to be used if teh availability of teh reference repository has to be tested

=cut

sub ref_adapter_pre_exec_string {
  my ( $self ) = @_;

  my $string = q{-E '} . $self->internal_script_bin() . q{/npg_pipeline_preexec_references};
  if ( $self->can( q{has_repository} ) && $self->has_repository() ) {
    $string .= q{ --repository } . $self->repository();
  }
  $string .= q{'};
  return $string;
}

=head2 files_present_pre_exec_string

pre-exec string to be used for testing the presence of complete_lane files

=cut

sub files_present_pre_exec_string {
  my ( $self ) = @_;

  my $command = q{-E '} . $self->internal_script_bin() . q{/npg_pipeline_files_present};
  my $join_string = q{ --file };
  my $archive_path = $self->runfolder_path() . q{/Latest_Summary/archive};
  if ( $self->no_summary_link() ) {
      $archive_path = $self->archive_path();
  }
  $join_string .= $archive_path . q{/completed_lane_};
  $command .= $join_string . ( join $join_string, $self->all_positions() ) . q{'};
  return $command;
}

=head2 tracking_dbix_connection_pre_exec_string

returns the string needed in a bsub command which runs the prexec check for a dbix connection to the tracking database

=cut

sub tracking_dbix_connection_pre_exec_string {
  my ( $self ) = @_;

  my $string = q{-E '} . $self->internal_script_bin() . q{/npg_pipeline_prexec_connect_dbix_tracking'};

  return $string;

}

=head2 pb_calibration_bin

returns the location of the pb_calibration scripts

  my $sPBCalibrationBin = $class->pb_calibration_bin();

=cut

has q{pb_calibration_bin} => (isa => q{Str}, is => q{ro}, lazy_build => 1,
  documentation => q{bin used for pb_calibration scripts},);

sub _build_pb_calibration_bin {
  my ( $self ) = @_;
  my $pb_cal_bin = $self->pb_cal_pipeline_conf()->{bin};
  if($pb_cal_bin =~ m{^/}smx){return $pb_cal_bin}
  return $self->filesystem_locations_conf()->{bin} . q{/} . $pb_cal_bin;
}

1;
__END__

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Moose::Role

=item Carp

=item English -no_match_vars

=item Readonly

=item st::api::lims

=item npg::api::run

=item npg_qc::autoqc::checks::tag_metrics

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

$Author: dj3 $

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by Andy Brown (ajb@sanger.ac.uk)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
