#############
# $Id: flag_options.pm 16184 2012-11-04 10:05:28Z dj3 $
# Created By: ajb
# Last Maintained By: $Author: dj3 $
# Created On: 2009-11-30
# Last Changed On: $Date: 2012-11-04 10:05:28 +0000 (Sun, 04 Nov 2012) $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-pipeline/tags/release-39.0/lib/npg_pipeline/roles/business/flag_options.pm $

package npg_pipeline::roles::business::flag_options;
use Moose::Role;

use Readonly; Readonly::Scalar our $VERSION => do { my ($r) = q$LastChangedRevision: 16184 $ =~ /(\d+)/mxs; $r; };

=head1 NAME

npg_pipeline::roles::business::flag_options

=head1 VERSION

$LastChangedRevision: 16184 $

=head1 SYNOPSIS

  package MyPackage;
  use Moose;
  ...
  with qw{npg_pipeline::roles::business::flag_options};

=head1 DESCRIPTION

This role gives some boolean flag options which should be set on construction (or via the command line if using MooseX::Getopt)
so that you can turn off global features/functions without having to necessarily specify them.

  --no_status_updates
  --no_folder_moves

These would globally stop anything being done should functions be requested which do these (either directly, or by job submission)

=head1 SUBROUTINES/METHODS

=head2 no_status_updates

Universally disables status_updates

=cut

has q{no_status_updates} => (
  isa => q{Bool},
  is => q{ro},
  documentation => q{Turn off updating any tracking statuses},
  lazy => 1,
  builder => '_default_to_local',
);

=head2 no_folder_moves

Universally disables anything that would move a folder

=cut

has q{no_folder_moves} => (isa => q{Bool}, is => q{ro},
  documentation => q{Turn off moving any folders},);

=head2 no_summary_link

Do not create a summary link

=cut

has q{no_summary_link} => (
  isa => q{Bool},
  is => q{ro},
  documentation => q{Turn off creating a Latest_Summary link},
  lazy => 1,
  builder => '_default_to_local',
);

=head2 no_control_lane

Act as though there is no control lane for recalibration purposes

=cut

has q{no_control_lane} => (isa => q{Bool}, is => q{ro}, lazy_build => 1,
  documentation => q{If you don't want any control lane recalibration, act as though there is no control lane},);

sub _build_no_control_lane {
  my ( $self ) = @_;
  return $self->no_recalibration();
}

=head2 no_recalibration

Act as though no recalibration should be done, regardless of the presence of a control lane or not
This will automatically set no_control_lane to true.

=cut

has q{no_recalibration} => (isa => q{Bool}, is => q{ro},
  documentation => q{Turn off doing any recalibration, regardless of control lane status}, );

=head2 no_spider

flag option to not spider out at all for webservice data

=cut

has q{no_spider} => (
  isa => q{Bool},
  is => q{ro},
  documentation => q{Turn off spidering and caching webservice data},
);

=head2 no_fix_config_files

flag option to request that config files are not checked and fixed

=cut

has q{no_fix_config_files} => (
  isa => q{Bool},
  is => q{ro},
  documentation => q{request that config files are not checked and fixed (where fixing is appropriate)},
);

=head2 no_array_cpu_limit

flag option to allow job arrays to flood, if able, the farm

=cut

has q{no_array_cpu_limit} => (
  isa => q{Bool},
  is => q{ro},
  documentation => q{Allow job arrays to keep launching if cpus available},
);

=head2 array_cpu_limit

set the most number of cpus which each job array can use at a time, applied only if no_array_cpu_limit not set

=cut

has q{array_cpu_limit} => (
  isa => q{Int},
  is => q{ro},
  documentation => q{Set the most number of CPUs that a Job array can use at a time},
  lazy_build => 1,
);

sub _build_array_cpu_limit {
  my ( $self ) = @_;
  return $self->general_values_conf()->{array_cpu_limit};
}

=head2 all_lanes_as_spiked_phix
=head2 all_lanes_as_non_spiked_phix

=cut

has q{all_lanes_as_spiked_phix} => (
  isa => q{Bool},
  is => q{ro},
  predicate => q{has_all_lanes_as_spiked_phix},
);


has q{all_lanes_as_non_spiked_phix} => (
  isa => q{Bool},
  is => q{ro},
  predicate => q{has_all_lanes_as_non_spiked_phix},
);

=head2 olb

Switches on OLB Bustard preprocessing to generate dif and bcl files and Bustard XML Summary from cif files; false by default.

=cut

has q{olb} => (
  isa => q{Bool},
  is => q{ro},
  documentation => q{Switches on OLD Bustard preprocessing to generate dif and bcl files and Bustard XML Summary from cif files; false by default.},
  default => 0,
);

=head2 no_irods_archival

Switches off archival to iRODS repository.

=cut

has q{no_irods_archival} => (
  isa => q{Bool},
  is => q{ro},
  documentation => q{Switches off archival to iRODS repository.},
  lazy => 1,
  builder => '_default_to_local',
);

## no critic (ProhibitUnusedPrivateSubroutines)
sub _default_to_local {
  my $self = shift;
  return $self->local;
}
## use critic

=head2 no_warehouse_update

Switches off updating the NPG warehouse.

=cut

has q{no_warehouse_update} => (
  isa => q{Bool},
  is => q{ro},
  documentation => q{Switches off updating the NPG warehouse.},
  lazy => 1,
  builder => '_default_to_local',
);

=head2 not_strip_bam_tag

not strip any tags in final bam file, like OQ, ci etc.

=cut

has q{not_strip_bam_tag} => (
  isa => q{Bool},
  is => q{ro},
  documentation => q{not strip any tags in final bam file, like OQ, ci etc.},
);

=head2 no_sf_resource

do not use sf resource tokens; set if working outside the npg sequencing farm

=cut

has q{no_sf_resource} => (
  isa => q{Bool},
  is => q{ro},
  documentation => q{do not use sf resource tokens; set if working outside the npg sequencing farm},
);

=head2 no_bsub

disable submitting any jobs to bsub, so the pipeline can be run, and all cmds logged

=cut

has q{no_bsub} => (
  isa => q{Bool},
  is => q{ro},
  documentation => q{Turn off submitting any jobs to lsf, just logging them instead},
);

=head2 local

sets the default for no_irods_archival, no_warehouse_update, no_status_updates and no_summary_link to true;
defaults to the value of no_bsub flag

=cut

has q{local} => (
  isa => q{Bool},
  is => q{ro},
  documentation => q{turn off lots of archiving and updating flags},
  lazy_build => 1,
);
sub _build_local {
  my $self = shift;
  return $self->no_bsub ? 1 : 0;
}

=head2 spatial_filter

Do we want to use the spatial_filter program?

=cut

has q{spatial_filter} => (
  isa => q{Bool},
  is => q{ro},
  documentation => q{use the spatial_filter program},
  default =>1,
);

1;
__END__

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Moose::Role

=item Carp

=item English -no_match_vars

=item Readonly

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

$Author: dj3 $

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by Andy Brown (ajb@sanger.ac.uk)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
