# $Id: 10-pluggable_harold.t 16101 2012-10-17 09:37:15Z mg8 $
use strict;
use warnings;
use Test::More tests => 56;
use Test::Deep;
use Test::Exception::LessClever;
use t::util;
use t::dbic_util;

my $util = t::util->new();
$util->catch_email();

$ENV{DBIC_TRACE} = 0;

$ENV{TEST_BIN} = q{t/bin/};
$ENV{TEST_DIR} = $util->temp_directory();
$ENV{OWNING_GROUP} = q{staff};
local $ENV{NPG_WEBSERVICE_CACHE_DIR} = q[t/data];

use_ok('npg_pipeline::pluggable::harold');

my $analysis_runfolder_path = $util->analysis_runfolder_path();
my $analysis_path = q{Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto};
my $schema = t::dbic_util->new->test_schema();

{
  my $harold;
  my @functions_in_order = qw(
      create_archive_directory
      create_fastq
      create_fastqcheck
      create_gcfreq
      create_md5
    );
  lives_ok {
    $harold = npg_pipeline::pluggable::harold->new({
      script_name => q{test},
      id_run => 1234,
      function_order => \@functions_in_order,
      run_folder => q{123456_IL2_1234},
      analysis_path => q{Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto},
      runfolder_path => $analysis_runfolder_path,
      conf_path => q{data/config_files},
    });
  } q{no croak on creation};
  isa_ok($harold, q{npg_pipeline::pluggable::harold}, q{$harold});
  push @functions_in_order, 'lsf_end';
  unshift @functions_in_order, 'lsf_start';
  is(join(q[ ], @{$harold->function_order()}), join(q[ ],@functions_in_order), q{function order set on creation and wrapped correctly});
  is($harold->pipeline_name, 'harold', 'pipeline name');
  lives_ok { $harold->parallelise(); } q{no croak obtaining parallelise_hash};

  no warnings 'once';
  foreach my $function (@npg_pipeline::pluggable::harold::CHANGE_STATUS_FUNCTIONS) {
    ok ($harold->can($function), qq{method $function is defined});
  }
  foreach my $function (@npg_pipeline::pluggable::harold::AUTOQC_FUNCTIONS) {
    ok ($harold->can($function), qq{method $function is defined});
  }
}

$util->set_staging_analysis_area({with_latest_summary => 1});
{
  my $harold = npg_pipeline::pluggable::harold->new(
      id_run => 1234,
      run_folder => q{123456_IL2_1234},
      domain => q{test},
      function_order => [],
      runfolder_path => $analysis_runfolder_path,
    );
  ok(!$harold->interactive, 'start job will be resumed');
  lives_ok { $harold->main() } 'main method with no functions defined runs ok';

  is(scalar @{$harold->dispatch_tree->functions}, 2, 'two functions');
  is($harold->dispatch_tree->functions->[0]->{function}, 'lsf_start', 'first function is start');
  is($harold->dispatch_tree->functions->[1]->{function}, 'lsf_end', 'second function is end');

  $harold = npg_pipeline::pluggable::harold->new(
      id_run => 1234,
      run_folder => q{123456_IL2_1234},
      domain => q{test},
      conf_path => q{data/config_files},
      runfolder_path => $analysis_runfolder_path,
      interactive => 1,
    );
  ok($harold->interactive, 'start job will not be resumed');
  lives_ok { $harold->main() } 'main method with no functions defined and no resume runs ok';
  is(scalar @{$harold->dispatch_tree->functions}, 2, 'two functions');
  local $ENV{NPG_WEBSERVICE_CACHE_DIR} = q[t/data];
  is (join( q[ ], $harold->positions), '1 2 3 4 5 6 7 8', 'positions array');
  is (join( q[ ], $harold->all_positions), '1 2 3 4 5 6 7 8', 'all positions array');

  my $function = 'touch_completed_lane';
  $harold = npg_pipeline::pluggable::harold->new(
      id_run => 1234,
      run_folder => q{123456_IL2_1234},
      domain => q{test},
      function_order => [$function],
      runfolder_path => $analysis_runfolder_path,
      lanes => [1,2],
      no_bsub => 1,
    );
  is(scalar @{$harold->dispatch_tree->functions}, 0, 'no functions');
  $harold->main;
  is(scalar @{$harold->dispatch_tree->functions}, 3, 'three functions');
  is($harold->dispatch_tree->functions->[0]->{function}, 'lsf_start', 'first function is start');
  is($harold->dispatch_tree->functions->[1]->{function}, $function, qq{first function is $function});
  is($harold->dispatch_tree->functions->[2]->{function}, 'lsf_end', 'third function is end');
  is($harold->dispatch_tree->functions->[2]->{job_dependencies}, q{-w'done(50) && done(50)'}, 'end dependencies');
  local $ENV{NPG_WEBSERVICE_CACHE_DIR} = q[t/data];
  is (join( q[ ], $harold->positions), '1 2', 'positions array');
  is (join( q[ ], $harold->all_positions), '1 2 3 4 5 6 7 8', 'all positions array');

  $harold = npg_pipeline::pluggable::harold->new(
      id_run => 1234,
      run_folder => q{123456_IL2_1234},
      domain => q{test},
      function_order => [$function, 'dodo'],
      runfolder_path => $analysis_runfolder_path,
      resume_start_job => 0,
      lanes => [1,2],
      no_bsub => 1,
  );
  throws_ok { $harold->main} qr/Error submitting jobs: Can't locate object method "dodo" via package "npg_pipeline::pluggable::harold"/, 'error when non-existing function is in the function order';
}

{
  my $harold;
  lives_ok {
    $harold = npg_pipeline::pluggable::harold->new({
      script_name => q{test},
      id_run => 6588,
      lanes => [1..8],
      run_folder => q{123456_IL2_1234},
      runfolder_path => $analysis_runfolder_path,
      npg_tracking_schema => $schema,
      no_bsub => 1,
      conf_path => $util->conf_path(),
      domain => q{test},
    });
  } q{no croak on creation};

  lives_ok {
    $harold->update_run_lane_analysis_in_progress();
  } q{run update_run_lane_analysis_in_progress ok - no dependencies};
  lives_ok {
    $harold->update_run_lane_analysis_complete({
      required_job_completion => q{-w'done(123) && done(321)'},
    });
  } q{run update_run_lane_analysis_complete ok - dependencies};
}

{
  my $harold = npg_pipeline::pluggable::harold->new({
      script_name => q{test},
      id_run => 6588,
      run_folder => q{123456_IL2_1234},
      no_bsub => 1,
      no_status_updates => 1,
      runfolder_path => $analysis_runfolder_path,
  });
  my @jids = $harold->update_run_lane_analysis_complete();
  is( scalar @jids, 0, q{no jobs submitted, as no_status_updates true} );
}

{
  my $qc;
  lives_ok {
    $qc = npg_pipeline::pluggable::harold->new({
      id_run => 1234,
      function_order => [qw(qc_qX_yield qc_insert_size)],
      run_folder => q{123456_IL2_1234},
      analysis_path => $analysis_path,
      runfolder_path => $analysis_runfolder_path,
      domain => q{test},
    });
  } q{no croak on creation};
  $util->set_staging_analysis_area({with_latest_summary => 1});
  isa_ok($qc, q{npg_pipeline::pluggable::harold}, '$qc');
  is(join(q[ ], @{$qc->function_order()}), 'lsf_start qc_qX_yield qc_insert_size lsf_end', 'function_order set on creation');
}

{
  local $ENV{NPG_WEBSERVICE_CACHE_DIR} = q[t/data];
  my $qc;
  $util->set_staging_analysis_area();
  lives_ok {
    $qc = npg_pipeline::pluggable::harold->new({
      script_name => q{test},
      id_run => 1234,
      run_folder => q{123456_IL2_1234},
      function_order => [qw{qc_qX_yield qc_adapter qc_insert_size}],
      lane => 4,
      analysis_path => $analysis_path,
      runfolder_path => $analysis_runfolder_path,
      domain => q{test},
      no_bsub => 1,
      repository => q{t/data/sequence},
    });
  } q{no croak on new creation};
  mkdir $qc->archive_path;
  mkdir $qc->qc_path;
  lives_ok { $qc->main() } q{no croak running qc->main()};
  local $ENV{NPG_WEBSERVICE_CACHE_DIR} = q[t/data];
  is (join( q[ ], $qc->positions), '4', 'positions array');
  is (join( q[ ], $qc->all_positions), '1 2 3 4 5 6 7 8', 'all positions array');
}

1;
